#include "addimagesdialog.h"
#include <iostream>

using namespace std;

QString AddImagesDialog::openFilesPath = QDir::currentPath();


AddImagesDialog::AddImagesDialog(QString title, QString applyString, QWidget *parent) :
        QDialog(parent)
{
    files.clear();
    setWindowTitle(title);

    QLabel* descriptionLabel = new QLabel("Create a list of the images you would like to add to the map");

    addButton = new QPushButton("&Add Image",this);
    addButton->setAutoDefault(false);
    removeButton = new QPushButton("&Remove Image",this);
    removeButton->setAutoDefault(false);
    removeAllButton = new QPushButton("Remove All",this);
    removeAllButton->setAutoDefault(false);
    applyButton = new QPushButton(applyString,this);

    forceCheck = new QCheckBox("Force");
    addEdgesCheck = new QCheckBox("Add Edges");
    addEdgesCheck->setChecked(true);

    imageList = new QListWidget(this);
    imageList->setMinimumWidth(500);


    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(addButton);
    vbox->addWidget(removeButton);
    vbox->addWidget(removeAllButton);
    vbox->addSpacing(20);
    vbox->addWidget(forceCheck);
    vbox->addSpacing(10);
    vbox->addWidget(addEdgesCheck);
    vbox->addStretch(1);

    QHBoxLayout *hbox = new QHBoxLayout;
    hbox->addWidget(imageList);
    hbox->addSpacing(10);
    hbox->addLayout(vbox);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addWidget(descriptionLabel);
    mainLayout->addLayout(hbox);
    mainLayout->addSpacing(20);
    mainLayout->addWidget(applyButton,0,Qt::AlignCenter);

    setLayout(mainLayout);

    updateButtons();

    connect(addButton, SIGNAL(clicked()), this, SLOT(add()));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(remove()));
    connect(removeAllButton, SIGNAL(clicked()), this, SLOT(removeAll()));
    connect(applyButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(this,SIGNAL(accepted()),this,SLOT(apply()));
    connect(imageList, SIGNAL(itemSelectionChanged()), this, SLOT(updateButtons()));

    addDialog = new QFileDialog(this);
    //addDialog.setViewMode(QFileDialog::List);
    addDialog->setViewMode(QFileDialog::Detail);
    addDialog->setFileMode(QFileDialog::ExistingFiles);
    addDialog->setWindowTitle("Add images...");
    addDialog->setNameFilter(QString("All image Files (*.ppm *.pgm)\nPPM images (*.ppm)\nPGM images (*.pgm)"));
    addDialog->setDirectory(openFilesPath);


    QPushButton *pb = new QPushButton;
    pb->setText("Add");
    pb->resize(QSize(40,20));
    connect(pb,SIGNAL(clicked()),this,SLOT(addToSidebar()));
    QLayout* l = addDialog->layout();
    QGridLayout* grid = dynamic_cast<QGridLayout*>(l);
    grid->addWidget(pb, 0, 0);
}


void AddImagesDialog::updateButtons()
{
    if(imageList->currentRow()>-1)
        removeButton->setEnabled(true);
    else
        removeButton->setEnabled(false);

    if(imageList->count()>0) {
        removeAllButton->setEnabled(true);
        applyButton->setEnabled(true);
        applyButton->setFocus();
    }
    else {
        removeAllButton->setEnabled(false);
        applyButton->setEnabled(false);
    }
}


void AddImagesDialog::add() {

    addDialog->exec();

    QStringList files = addDialog->selectedFiles();
    if (files.count()) {
        foreach(QString s, files) {
            QFileInfo fi(s);
            openFilesPath = fi.absolutePath();
            QList<QListWidgetItem*> found = imageList->findItems(s,Qt::MatchExactly);
            if(found.count()==0) {
                imageList->addItem(s);
            }
        }
        updateButtons();
    }
}


void AddImagesDialog::addToSidebar()
{
    QList<QUrl> urls = addDialog->sidebarUrls();
    urls << QUrl::fromLocalFile(addDialog->directory().absolutePath());
    addDialog->setSidebarUrls(urls);
}


void AddImagesDialog::remove() {
    imageList->takeItem(imageList->currentRow());
    updateButtons();
}


void AddImagesDialog::removeAll() {
    imageList->clear();
    updateButtons();
}

// add the list of images...
void AddImagesDialog::apply() {
    for(int i=0; i<imageList->count(); i++) {
        QListWidgetItem* file = imageList->item(i);
        files.push_back(file->text());
    }
}

