// Implementation of Astar search algorithm

#ifndef ASTAR_H
#define ASTAR_H

#include <iostream>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/astar_search.hpp>
#include "defs.h"
#include "graph.h"

using namespace std;
using namespace boost;



class AStarHeuristic : public boost::astar_heuristic<Graph, double>
{
public:
	double operator()( vertexDescriptor vertex ) const {return 0.0;}
};

// exception for termination
struct FoundGoal {};


//
// A* Visitor that terminates when we reach the goal
//
class AStarVisitor : public boost::default_astar_visitor
{
public:
	AStarVisitor( const vertexDescriptor &goal )
		: goal_(goal) {}

	template <class Graph>
    void examine_vertex( vertexDescriptor vertex, Graph &g )
	{
		if( vertex == goal_ )
		throw FoundGoal();
	}
private:
	vertexDescriptor goal_;
};


class AStar
{
private:
	AStarHeuristic aStarHeuristic;

public:
	AStar() {};
	bool searchThroughGraph( const Graph &graph, vertexDescriptor start, vertexDescriptor goal, vector<vertexDescriptor> &shortestPath )
	{
		AStarVisitor aStarVisitor( goal );

        //Graph &nonConstGraph = const_cast<Graph&>(graph);
        //typedef boost::property_map<Graph, double Edge::*>::type EdgeWeightMap;
        //EdgeWeightMap edgeWeightMap = get(&Edge::cost, nonConstGraph);
		vector<vertexDescriptor> predecessors(num_vertices(graph));

		try {
            //astar_search( graph, start, aStarHeuristic, boost::weight_map(edgeWeightMap).predecessor_map(&predecessors[0]).visitor(aStarVisitor) );
            astar_search( graph, start, aStarHeuristic, predecessor_map(&predecessors[0]).visitor(aStarVisitor) );
		}
		catch ( const FoundGoal& )
        {
            shortestPath.clear();
			for ( vertexDescriptor vertex = goal; ; vertex = predecessors[vertex] )
			{
				shortestPath.push_back( vertex );
				if ( predecessors[vertex]==vertex ) break;
            }

			return true;
		}

		// If we get to here, we couldn't reach the goal
		return false;
	}
};

#endif
