#include "changeconfigdialog.h"

ChangeConfigDialog::ChangeConfigDialog(QWidget *parent) : QDialog(parent)
{
    setWindowTitle(QString("Change Configuration..."));

    QIntValidator *ivalidator = new QIntValidator();

    minmatchesInput = new QLineEdit();
    minmatchesInput->setValidator(ivalidator);
    minmatchesInput->setMaximumSize(60,25);
    minmatchesInput->setMaxLength(5);
    minmatchesInput->setCursorPosition(0);
    minmatchesInput->setVisible(true);

    maxmatchesInput = new QLineEdit();
    maxmatchesInput->setValidator(ivalidator);
    maxmatchesInput->setMaximumSize(60,25);
    maxmatchesInput->setMaxLength(5);
    maxmatchesInput->setCursorPosition(0);
    maxmatchesInput->setVisible(true);

    gridmapInput = new QLineEdit();
    gridmapInput->setVisible(true);

    QPushButton* browseButton = new QPushButton("Browse...");
    browseButton->setAutoDefault(false);

    updateButton = new QPushButton("&Update",this);
    updateButton->setAutoDefault(true);

    cancelButton = new QPushButton("&Cancel",this);
    cancelButton->setAutoDefault(false);

    connect(browseButton, SIGNAL(clicked()), this, SLOT(browse()));
    connect(updateButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(close()));

    myDialog = new QFileDialog(this);
    myDialog->setViewMode(QFileDialog::List);
    myDialog->setFileMode(QFileDialog::ExistingFile);
    myDialog->setWindowTitle("Choose gridmap...");
    myDialog->setNameFilter(QString("All image Files (*.png *.ppm *.pgm)\nPNG images (*.png)\nPPM images (*.ppm)\nPGM images (*.pgm)"));
    myDialog->setDirectory(QCoreApplication::applicationDirPath().append("/sys"));

    QHBoxLayout *hboxmin = new QHBoxLayout;
    QLabel* minmatchesLabel = new QLabel("Number of Min Matches");
    hboxmin->addWidget(minmatchesLabel);
    hboxmin->addStretch(1);
    hboxmin->addWidget(minmatchesInput);

    QHBoxLayout *hboxmax = new QHBoxLayout;
    QLabel* maxmatchesLabel = new QLabel("Number of Max Matches");
    hboxmax->addWidget(maxmatchesLabel);
    hboxmax->addStretch(1);
    hboxmax->addWidget(maxmatchesInput);

    QHBoxLayout *hboxgridmap = new QHBoxLayout;
    QLabel* gridmapLabel = new QLabel("Gridmap");
    hboxgridmap->addWidget(gridmapLabel);
    hboxgridmap->addStretch(1);
    hboxgridmap->addWidget(gridmapInput);
    hboxgridmap->addStretch(1);
    hboxgridmap->addWidget(browseButton);

    QHBoxLayout *hboxbutton = new QHBoxLayout;
    hboxbutton->addWidget(updateButton);
    hboxbutton->addSpacing(10);
    hboxbutton->addWidget(cancelButton);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addLayout(hboxmin);
    vbox->addLayout(hboxmax);
    vbox->addLayout(hboxgridmap);
    vbox->addSpacing(15);
    vbox->addLayout(hboxbutton);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addLayout(vbox);

    setLayout(mainLayout);

    setFixedSize(QSize(350,150));
}


void ChangeConfigDialog::browse()
{
    myDialog->exec();
    QStringList files = myDialog->selectedFiles();
    if (files.count()) {
        setGridmap(files.at(0));
    }
}
