#ifndef DICTIONARY_H
#define DICTIONARY_H

#include <vector>
#include <string>
#include "defs.h"
#include "image.h"
#include "index.h"


typedef boost::unordered_map<imageIndex,PointList> OccuranceList;
typedef boost::unordered_map<wordIndex,OccuranceList> InvIndex; // map<wordIndex,occurances>


class Dictionary
{
public:
    Dictionary(string clusterfile, string indexfile);
    ~Dictionary();

    //! Clears the inverted index
    void clear();

    InvIndex* getInvIndex() { return &invIndex; }
    bool isEmpty()          { return invIndex.empty(); }
    int size()              { return invIndex.size(); }
    int numWords()          { return index->getClusters()->rows; }


    void addImage(Image *img);
    void findMatches(Image *img, PointMatchMap &matches);
    void queryDictionary(Image *img, Mat &indices, Mat &distances);

    //! Printout all the occurances in the dictionary
    void print();

    void writeDictionary(string filename);
    void readDictionary(string filename,ImageTable &imageLookup);


private:  
    //! Holds all visual words in the trained dictonary
    cvflannIndex::Index<FT,CT> *index;

    //! Holds all visual word occurances
    InvIndex invIndex;

    void writeInvIndex(string filename);
    void readInvIndex(string filename, ImageTable &imageLookup);
};

#endif // DICTIONARY_H
