#include "imagecomparisonconfig.h"

#include <QtGui>

#include <iostream>
using namespace std;

ImageComparisonConfig::ImageComparisonConfig(QWidget *parent)
    : QWidget(parent)
{
    subwin = NULL;

    QFont font;
    font.setPixelSize(11);
    setFont(font);

    setWindowTitle(tr("Configuration"));

    //applyButton = new QPushButton("Apply");

    QGridLayout *grid = new QGridLayout;
    grid->addWidget(createMethodGroup(), 0, 0);
    grid->addWidget(createFilterGroup(), 0, 1);
    //grid->addWidget(applyButton,1,0,1,2, Qt::AlignCenter);
    setLayout(grid);
    layout()->setSizeConstraint(QLayout::SetFixedSize);
    layout()->setMargin(5);

    connect(radioEdge,SIGNAL(clicked()),this,SLOT(updateVisual()));
    connect(radioPair,SIGNAL(clicked()),this,SLOT(updateVisual()));
    connect(radioCustom,SIGNAL(clicked()),this,SLOT(updateVisual()));
    //connect(applyButton,SIGNAL(clicked()),this,SIGNAL(buttonPressed()));

    connect(radioEdge,SIGNAL(clicked()),this,SIGNAL(configChanged()));
    connect(radioPair,SIGNAL(clicked()),this,SIGNAL(configChanged()));
    connect(radioCustom,SIGNAL(clicked()),this,SIGNAL(configChanged()));

    connect(checkBoxDup,SIGNAL(clicked()),this,SIGNAL(configChanged()));
    connect(checkBoxFund,SIGNAL(clicked()),this,SIGNAL(configChanged()));
    connect(checkBoxOut,SIGNAL(clicked()),this,SIGNAL(configChanged()));
    connect(checkBoxSpa,SIGNAL(clicked()),this,SIGNAL(configChanged()));

    updateVisual();

    //applyButton->setDefault(true);
    //applyButton->setFocus();
}


void ImageComparisonConfig::closeEvent(QCloseEvent *) {

    //cout<<"image Comparison CONFIG closeevent"<<endl;
    emit closeSignal();
    //cout<<"image Comparison CONFIG closeevent DONE"<<endl;
}

void ImageComparisonConfig::close()
{
    //cout<<"image Comparison CONFIG close"<<endl;
    if(subwin!=NULL) {
        subwin->mdiArea()->removeSubWindow(subwin);
        subwin->close();
    }
    //cout<<"image Comparison CONFIG close DONE"<<endl;
}


void ImageComparisonConfig::updateVisual()
{
    if(radioCustom->isChecked()) {
        checkBoxDup->setChecked(true);
        groupBoxFilter->setEnabled(true);
    }
    else {
        checkBoxDup->setChecked(false);
        groupBoxFilter->setEnabled(false);
    }
}

QGroupBox* ImageComparisonConfig::createMethodGroup()
{
    groupBoxMethod = new QGroupBox(tr("Computation of matches"));

    radioEdge = new QRadioButton(tr("&Show edge content"));
    radioPair = new QRadioButton(tr("&Pairwise comparison"));
    radioCustom = new QRadioButton(tr("&Dictionary && customization"));
    radioEdge->setChecked(true);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(radioEdge);
    vbox->addWidget(radioPair);
    vbox->addWidget(radioCustom);
    vbox->setSpacing(6);
    vbox->setMargin(2);
    vbox->addStretch(1);
    groupBoxMethod->setLayout(vbox);
    return groupBoxMethod;
}



QGroupBox* ImageComparisonConfig::createFilterGroup()
{
    groupBoxFilter = new QGroupBox(tr("Filters"));

    checkBoxDup = new QCheckBox(tr("Remove &duplicates"));
    checkBoxFund = new QCheckBox(tr("&Fundamental matrix"));
    checkBoxOut = new QCheckBox(tr("Remove &outliers"));
    checkBoxSpa = new QCheckBox(tr("&Spatial consistency"));
    checkBoxDup->setChecked(true);
    checkBoxDup->setEnabled(false);

    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(checkBoxDup);
    vbox->addWidget(checkBoxFund);
    vbox->addWidget(checkBoxOut);
    vbox->addWidget(checkBoxSpa);
    vbox->setSpacing(1);
    vbox->setMargin(2);
    vbox->addStretch(1);
    groupBoxFilter->setLayout(vbox);

    return groupBoxFilter;
}


int ImageComparisonConfig::getMethod()
{
    int val = 0;
    if(radioEdge->isChecked())
        val = 0;
    if(radioPair->isChecked())
        val = 1;
    if(radioCustom->isChecked())
        val = 2;
    return val;
}

int ImageComparisonConfig::getFilter()
{
    int filter = 0b000;
    if(checkBoxFund->isChecked())
        filter = filter | 0b001;
    if(checkBoxOut->isChecked())
        filter = filter | 0b010;
    if(checkBoxSpa->isChecked())
        filter = filter | 0b100;

    return filter;
}
