#ifndef IMAGECOMPARISONWIDGET_H
#define IMAGECOMPARISONWIDGET_H

#include "image.h"
#include "simmap.h"
#include "imagecomparisonconfig.h"

#include <QWidget>
#include <QLabel>
#include <QAction>
#include <QMdiArea>

//! Widget to compare two images using selected method and modules. It visualizes the matches
class ImageComparisonWidget : public QWidget
{
    Q_OBJECT
public:
    // @TODO: The object will get a copy of the pointer NOT the actual Images/Edges
    // So if the image or the edge changes, comparison window will not work
    ImageComparisonWidget(Image*, Image*, int orientation=1, Edge *e=NULL, SimMap *m=NULL, QWidget *parent=0);
    void drawMatches(int method=0, int filter=0b001);

    void setSubWindowPtr(QMdiSubWindow *s);

signals:
    void closeSignal();

protected:
    void contextMenuEvent(QContextMenuEvent *event);
    void closeEvent(QCloseEvent *);
private slots:
    void showInfo();
    void showConfWindow();
    void killConfWindow();
    void close();
    void updateDisplay();
private:
    QLabel *image1Label;
    QLabel *image2Label;

    Image *image1;
    Image *image2;
    Edge *edge;
    SimMap *mymap;
    QMdiSubWindow *subwin;
    QMdiSubWindow *confsubwin;

    QPixmap *pix1;
    QPixmap *pix2;
    QPixmap *pix1_org;
    QPixmap *pix2_org;

    QStatusBar *statusBar;

    // if the image is landscape or horizontal
    bool isVertical;

    QAction *infoAct;
    QAction *confAct;

    bool openImages();
    bool openImage(Image*,QPixmap&);
    void updateStatusBar();

    int numMatches;

    ImageComparisonConfig *confwin;
};

#endif // IMAGECOMPARISONWIDGET_H
