#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "imagewindow.h"
#include <QMainWindow>
#include <QMdiArea>
#include <QLabel>
#include <QAction>

class MainWindow : public QMainWindow
{
    Q_OBJECT;
public:
    MainWindow(QMainWindow* parent=0);

protected:
    void closeEvent(QCloseEvent *);

private slots:
    void updateActions();

    void about();
    void openImage();
    void saveImage();
    void saveAsImage();
    void compareImages();

    void openMap();
    void saveMap();
    void saveAsMap();
    void exportMap();
    void buildMap();
    void addImages();
    void addEdge();
    void removeVertex();
    void removeEdge();
    void removeEdges();
    void removeConnectedComp();
    void test(); // reserved test function

    void closeAllWindows();
    void closeAllImageWindows();
    void tileWindows();

    void disableAllInputs();
    void enableAllInputs();

    void changeConfig();

private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();

    QMdiArea *mdiArea;

    // MENUS
    QMenu *fileMenu; // open, save, save as workspace...
    QMenu *mapMenu;
    QMenu *imageMenu;
    QMenu *windowMenu;
    QMenu *helpMenu;

    // TOOLBARS
    QToolBar *fileToolBar;
    QToolBar *aboutToolBar;
    QToolBar *mapToolBar;
    QToolBar *imageToolBar;

    // ACTIONS
    QAction *exitAction;
    QAction *aboutAction;
    QAction *aboutQtAction;
    QAction *changeConfigAction;

    QAction *openImageAction;
    QAction *saveImageAction;
    QAction *saveAsImageAction;
    QAction *compareImageAction;

    QAction *openMapAction;
    QAction *saveMapAction;
    QAction *saveAsMapAction;
    QAction *exportMapAction;
    QAction *buildMapAction;
    QAction *addImagesAction;
    QAction *addEdgeAction;
    QAction *delVertexAction;
    QAction *delEdgeAction;
    QAction *delComponentAction;
    QAction *testAction;

    QAction *closeAllWindowsAction;
    QAction *closeAllImageWindowsAction;
    QAction *tileWindowsAction;

    //QString workingDirectory;
};

#endif // MAINWINDOW_H
