#include "icons.h"
#include "mapwidget.h"
#include "imagecomparisonwidget.h"
#include "progressbar.h"
#include "vertexinput.h"
#include "edgeinput.h"
#include <iostream>
#include <QMdiArea>
#include <QMessageBox>
#include <QFileInfo>
#include <QMenu>
#include <QtGui>
#include <boost/thread/thread.hpp>

using namespace std;

MapWidget::MapWidget(QWidget *parent)
    : QWidget(parent)
{
    simMap = NULL;
    filename = "";
    title = "";
    mdiArea = NULL;

    mapLabel = new QLabel(this);   
    mapLabel->setBackgroundRole(QPalette::Base);
    mapLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    mapLabel->setScaledContents(true);
    pixmap = NULL;
    painter = new QPainter();

    scrollArea = new QScrollArea(this);
    scrollArea->setBackgroundRole(QPalette::Dark);
    scrollArea->setWidget(mapLabel);
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    createActions();

    //dispMode = SimGraph::GRIDMAPMODE;
    //dispMode = SimGraph::MATRIXMODE;
    dispMode = SimGraph::CIRCMODE;
    coloring = "clusterId";

    processStop = false;

    connect(qApp,SIGNAL(destroyed()),this,SLOT(close()));

    updateActions();
}


MapWidget::MapWidget(QString fname, QWidget *parent)
    : QWidget(parent)
{
    simMap = new SimMap(fname.toStdString());

    filename = fname;
    QFileInfo fi(filename);
    setTitle(fi.fileName());

    mdiArea = NULL;

    mapLabel = new QLabel(this);
    mapLabel->setBackgroundRole(QPalette::Base);
    //mapLabel->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    mapLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    mapLabel->setScaledContents(true);
    pixmap = NULL;
    painter = new QPainter();

    scrollArea = new QScrollArea(this);
    scrollArea->setBackgroundRole(QPalette::Dark);
    scrollArea->setWidget(mapLabel);
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded); 

    createActions();

    //dispMode = SimGraph::GRIDMAPMODE;
    //dispMode = SimGraph::MATRIXMODE;
    dispMode = SimGraph::CIRCMODE;
    coloring = "clusterId";

    processStop = false;

    connect(qApp,SIGNAL(destroyed()),this,SLOT(close()));
    updateActions();
}



void MapWidget::closeEvent(QCloseEvent *)
{
    cout<<"close event for MapWidget"<<endl;
    emit closing();
    if(simMap!=NULL) {
        delete simMap;
        simMap = NULL;
    }
    //cout<<"close event for MapWidget-> DONE"<<endl;
}

void MapWidget::stop(){
    processStop = true;
}


void MapWidget::createActions() {
    mapInfoAct = new QAction(tr("Map &Info"), this);
    mapInfoAct->setShortcut(tr("Ctrl+I"));
    mapInfoAct->setEnabled(true);
    connect(mapInfoAct, SIGNAL(triggered()), this, SLOT(mapInfo()));

    computeScoreAct = new QAction(tr("&Compute Score"),this);
    computeScoreAct->setStatusTip(tr("Compute Score"));
    computeScoreAct->setEnabled(true);
    connect(computeScoreAct,SIGNAL(triggered()),this,SLOT(computeScore()));

    showTimingAct = new QAction(tr("Print &Timing"),this);
    showTimingAct->setStatusTip(tr("Print Timing"));
    showTimingAct->setIcon(QIcon(ICON_STATS));
    connect(showTimingAct,SIGNAL(triggered()),this,SLOT(showTiming()));

    statsToFileAct = new QAction(tr("&Write Stats to File"),this);
    statsToFileAct->setStatusTip(tr("Write Stats to File"));
    statsToFileAct->setIcon(QIcon(ICON_STATS));
    connect(statsToFileAct,SIGNAL(triggered()),this,SLOT(writeStatsToFile()));

    simMatrixToFileAct = new QAction(tr("&Write SimMatrix to File"),this);
    simMatrixToFileAct->setStatusTip(tr("Write SimMatrix to File"));
    //statsToFileAct->setIcon(QIcon(ICON_STATS));
    connect(simMatrixToFileAct,SIGNAL(triggered()),this,SLOT(writeSimMatrixToFile()));

    gridmapModeAct = new QAction(tr("Switch To Gridmap Mode"),this);
    gridmapModeAct->setStatusTip(tr("Switch To Gridmap Mode"));
    gridmapModeAct->setShortcut(tr("Ctrl+M"));
    gridmapModeAct->setEnabled(true);
    connect(gridmapModeAct,SIGNAL(triggered()),this,SLOT(switchToGridmapMode()));

    graphModeAct = new QAction(tr("Switch To Graph Mode"),this);
    graphModeAct->setStatusTip(tr("Switch To Graph Mode"));
    graphModeAct->setShortcut(tr("Ctrl+G"));
    graphModeAct->setEnabled(true);
    connect(graphModeAct,SIGNAL(triggered()),this,SLOT(switchToGraphMode()));

    circModeAct = new QAction(tr("Switch To Circ Mode"),this);
    circModeAct->setStatusTip(tr("Switch To Circ Mode"));
    circModeAct->setShortcut(tr("Ctrl+C"));
    circModeAct->setEnabled(true);
    connect(circModeAct,SIGNAL(triggered()),this,SLOT(switchToCircMode()));

    matrixModeAct = new QAction(tr("Switch To Matrix Mode"),this);
    matrixModeAct->setStatusTip(tr("Switch To Matrix Mode"));
    matrixModeAct->setShortcut(tr("Ctrl+C"));
    matrixModeAct->setEnabled(true);
    connect(matrixModeAct,SIGNAL(triggered()),this,SLOT(switchToMatrixMode()));

    resetVertexLabelsAct = new QAction(tr("Reset Vertex Labels"),this);
    resetVertexLabelsAct->setStatusTip(tr("Reset Vertex Labels"));
    resetVertexLabelsAct->setEnabled(true);
    connect(resetVertexLabelsAct,SIGNAL(triggered()),this,SLOT(resetVertexLabels()));

    detachWidgetAct = new QAction(tr("Detach Widget"),this);
    detachWidgetAct->setStatusTip(tr("Detach Widget"));
    detachWidgetAct->setEnabled(true);
    connect(detachWidgetAct,SIGNAL(triggered()),this,SLOT(detachWidget()));

    attachWidgetAct = new QAction(tr("Attach Widget"),this);
    attachWidgetAct->setStatusTip(tr("Attach Widget"));
    attachWidgetAct->setEnabled(false);
    connect(attachWidgetAct,SIGNAL(triggered()),this,SLOT(attachWidget()));

    colorByMapId = new QAction(tr("Color by mapId"),this);
    colorByMapId->setStatusTip(tr("Color by mapId"));
    colorByMapId->setEnabled(true);

    colorByClusterId = new QAction(tr("Color by clusterId"),this);
    colorByClusterId->setStatusTip(tr("Color by clusterId"));
    colorByClusterId->setEnabled(true);

    colorByCustomVar = new QAction(tr("Color by customVar"),this);
    colorByCustomVar->setStatusTip(tr("Color by customVar"));
    colorByCustomVar->setEnabled(true);

    colorVertex = new QAction(tr("Color vertex"),this);
    colorVertex->setStatusTip(tr("Color vertex"));
    colorVertex->setEnabled(true);

    colorEdge = new QAction(tr("Color edge"),this);
    colorEdge->setStatusTip(tr("Color edge"));
    colorEdge->setEnabled(true);

    QSignalMapper* signalMapper = new QSignalMapper (this);
    connect (colorByMapId, SIGNAL(triggered()), signalMapper, SLOT(map()));
    connect (colorByClusterId, SIGNAL(triggered()), signalMapper, SLOT(map()));
    connect (colorByCustomVar, SIGNAL(triggered()), signalMapper, SLOT(map()));
    connect (colorVertex, SIGNAL(triggered()), signalMapper, SLOT(map()));
    connect (colorEdge, SIGNAL(triggered()), signalMapper, SLOT(map()));

    // different coloring options based on different criteria
    signalMapper -> setMapping (colorByMapId, "mapId");
    signalMapper -> setMapping (colorByClusterId, "clusterId");
    signalMapper -> setMapping (colorByCustomVar, "customVar");
    signalMapper -> setMapping (colorVertex, "vertex");
    signalMapper -> setMapping (colorEdge, "edge");
    connect (signalMapper, SIGNAL(mapped(QString)), this, SLOT(changeColoring(QString)));

}


void MapWidget::updateActions() {
    if(dispMode==SimGraph::GRAPHMODE)
        graphModeAct->setEnabled(false);
    else
        graphModeAct->setEnabled(true);

    if(dispMode==SimGraph::GRIDMAPMODE)
        gridmapModeAct->setEnabled(false);
    else
        gridmapModeAct->setEnabled(true);

    if(dispMode==SimGraph::CIRCMODE)
        circModeAct->setEnabled(false);
    else
        circModeAct->setEnabled(true);

    if(dispMode==SimGraph::MATRIXMODE)
        matrixModeAct->setEnabled(false);
    else
        matrixModeAct->setEnabled(true);
}

void MapWidget::mapInfo() {
    stringstream descr;
    descr << "Filename: " << filename.toStdString() << endl;

    if(simMap!=NULL) {
        descr << "Number of vertices: " << simMap->numVertices() << endl;
        descr << "Number of edges: " << simMap->numEdges() << endl;
        descr << "Min vertex degree: " << simMap->graphMinDegree() << endl;
        descr << "Max vertex degree: " << simMap->graphMaxDegree() << endl;
        descr << "Avg vertex degree: " << simMap->graphAvgDegree() << endl;
        descr << "Number of connected components: " << simMap->graphNumConnComp() << endl;
    }

    QMessageBox::about(this, tr("Map Info"),QString(descr.str().c_str()));
}

void MapWidget::writeStatsToFile() {
    // no need for a dialog right now
    //QString filename = QFileDialog::getSaveFileName(this, "Save Stats...",".","All files (*.*)");
    if(simMap!=NULL) {
        QString filename("stats");
        //simMap->getGraph()->writeStats(filename.toStdString());
        simMap->writeStats(filename);
    }
}

void MapWidget::showTiming()
{
    if(simMap!=NULL) {
        // @TODO: write a function in simmap to time each slam process
        // for debugging purposes...
    }
}

void MapWidget::computeScore() {
    if(simMap!=NULL) {
        double score = simMap->scoreMap();
        QMessageBox::about(this, tr("Score"),QString("Score: %1").arg(score));
    }
}


QString MapWidget::getFilename() {
    return filename;
}

void MapWidget::setFilename(QString fname) {
    filename = fname;
    QFileInfo fi(filename);
    this->setWindowTitle(fi.fileName());
}


QString MapWidget::getTitle() {
    return title;
}

void MapWidget::setTitle(QString t) {
    title = t;
}


void MapWidget::mousePressEvent (QMouseEvent * event)
{
    //cout<<mdiArea->size().height()<<" "<<mdiArea->size().width()<<endl;

    if(event->button()==Qt::LeftButton) {
        if(simMap!=NULL && !simMap->isEmpty()) {
            // Coordinate frames are different so we change the direction of y
            QScrollBar *vert = scrollArea->verticalScrollBar();
            QScrollBar *horz = scrollArea->horizontalScrollBar();
            int x,y;
            if(dispMode == SimGraph::GRAPHMODE) {
                x = event->x() + horz->value();
                y = mapLabel->size().height()- (event->y() + vert->value()); // needed for graphviz output images
            }
            else {
                x = event->x() + horz->value();
                y = event->y() + vert->value();
            }

            double vdist;
            double edist;
            Vertex *v = simMap->getGraph()->getClosestVertex(x,y,vdist);
            Edge *e = simMap->getGraph()->getClosestEdge(x,y,edist);

            if(vdist <= edist) {
                bool objectFound = false;
                if(dispMode==SimGraph::GRAPHMODE)
                    objectFound = v->graphPose.isInside(x,y);
                else if(dispMode==SimGraph::GRIDMAPMODE)
                    objectFound = v->gridmapPose.isInside(x,y);
                else if(dispMode==SimGraph::CIRCMODE)
                    objectFound = v->circPose.isInside(x,y);

                if(objectFound)
                    showVertex(v);
            }
            else {
                bool objectFound = false;
                if(dispMode==SimGraph::GRAPHMODE)
                    objectFound = e->graphPose.isInside(x,y);
                else if(dispMode==SimGraph::GRIDMAPMODE)
                    objectFound = e->gridmapPose.isInside(x,y);
                else if(dispMode==SimGraph::CIRCMODE)
                    objectFound = e->circPose.isInside(x,y);

                if (objectFound)
                    showEdge(e);
            }

            /*
            if(vdist <= edist) {
                if(v!=NULL)
                    showVertex(v,x,y);
                else if(e!=NULL)
                    showEdge(e,x,y);
            }
            else {
                if(e!=NULL)
                    showEdge(e,x,y);
                else if(v!=NULL)
                    showVertex(v,x,y);
            }
            */
        }
    }
    else if(event->button()==Qt::RightButton) {
        updateActions();

        QMenu contextMenu(this);
        contextMenu.addAction(mapInfoAct);
        contextMenu.addAction(computeScoreAct);
        contextMenu.addAction(resetVertexLabelsAct);

        //QMenu *coloring = contextMenu.addMenu("Coloring");
        //coloring->addAction(colorByMapId);
        //coloring->addAction(colorByClusterId);
        //coloring->addAction(colorByCustomVar);
        //coloring->addSeparator();
        //coloring->addAction(colorVertex);
        //coloring->addAction(colorEdge);

        contextMenu.addSeparator();
        contextMenu.addAction(colorByMapId);
        contextMenu.addAction(colorByClusterId);
        contextMenu.addAction(colorByCustomVar);
        contextMenu.addAction(colorVertex);
        contextMenu.addAction(colorEdge);

        contextMenu.addSeparator();
        contextMenu.addAction(statsToFileAct);
        contextMenu.addAction(showTimingAct);
        contextMenu.addAction(simMatrixToFileAct);

        contextMenu.addSeparator();
        contextMenu.addAction(graphModeAct);
        contextMenu.addAction(circModeAct);
        contextMenu.addAction(gridmapModeAct);
        contextMenu.addAction(matrixModeAct);
        contextMenu.addSeparator();

        if(attachWidgetAct->isEnabled())
            contextMenu.addAction(attachWidgetAct);
        if(detachWidgetAct->isEnabled())
            contextMenu.addAction(detachWidgetAct);

        contextMenu.exec(event->globalPos()+QPoint(3,3));
    }
}


// open image corresponding to the selected vertex
void MapWidget::showVertex(Vertex *v)
{
    if(v!=NULL) {
        Debug(1)<<"clicked vertex: "<<v->index<<endl;
        ImageWidget* selImage = new ImageWidget();
        selImage->openImage(v->image);
        string wintitle = "Vertex: "+v->label;
        if(v->image->getTitle() != "")
            wintitle = wintitle + ", " + v->image->getTitle();
        selImage->setWindowTitle(wintitle.c_str());
        if(mdiArea!=NULL) {
            //if(mdiArea->activeSubWindow())
            mdiArea->addSubWindow(selImage);
        }
        selImage->show();
    }
}

// show two images side by side corresponding to the selected edge
void MapWidget::showEdge(Edge *e)
{
    if(e!=NULL) {
        Debug(1)<<"clicked edge: "<<e->v1<<"-"<<e->v2<<endl;
        Image *img1 = (*simMap->getImages())[e->v1];
        Image *img2 = (*simMap->getImages())[e->v2];
        ImageComparisonWidget* compImage = new ImageComparisonWidget(img1,img2,1,e,simMap);
        string wintitle = "Comparison: "+converter::int2str(img1->getIndex())+" vs "+converter::int2str(img2->getIndex());
        compImage->setWindowTitle(wintitle.c_str());
        if(mdiArea!=NULL) {
            QMdiSubWindow *s = mdiArea->addSubWindow(compImage);
            compImage->setSubWindowPtr(s);
        }
        compImage->show();
        connect(this,SIGNAL(closing()),compImage,SLOT(close()));
    }
}


void MapWidget::resizeEvent(QResizeEvent *event) {
    //    cout<<"resized: "<<event->size().height()<<" "<<event->size().width()<<endl;
    resize(event->size());
    scrollArea->resize(event->size());
}

void MapWidget::setMdiPtr(QMdiArea* m)
{
    mdiArea = m;
    setMaximumSize(mdiArea->size());
    scrollArea->setMaximumSize(mdiArea->size()-QSize(8,28));
}


void MapWidget::changeColoring(QString s) {
    coloring = s;

    if(coloring=="vertex")
    {
        VertexInput *vin = new VertexInput();
        vin->show();
        vin->exec();
        int v = vin->getVertexId();
        Vertex *v1 = simMap->getGraph()->getVertex(v);
        if(v1!=NULL) {
            QPen pen(painter->pen());
            pen.setColor(QColor("red"));
            pen.setWidth(1);

            pair<adjacencyIterator, adjacencyIterator> viter = simMap->getGraph()->getEdges(v);
            adjacencyIterator it;
            for(it=viter.first; it!=viter.second; it++)
            {
                Vertex *v2 = simMap->getGraph()->getVertex(it);

                if(v1->mapId!=v2->mapId) {
                    cout<<"edge between: "<<v1->index<<" "<<v2->index<<endl;

                    Edge *e = simMap->getGraph()->getEdge(v1->index,v2->index);
                    painter->setPen(pen);
                    drawEdge(e,dispMode,true);
                }
            }
            painter->setPen(pen);
            painter->setBrush(QColor("red"));
            drawVertex(v1,dispMode);

            mapLabel->setPixmap(*pixmap);
            mapLabel->adjustSize();
            scrollArea->resize(mapLabel->size());
            adjustSize();

            showVertex(v1);
        }
    }
    else if(coloring=="edge")
    {
        EdgeInput *ein = new EdgeInput();
        ein->show();
        ein->exec();
        int v1 = ein->getVertex1Id();
        int v2 = ein->getVertex2Id();
        if(v1>0 && v2>0) {
            QPen pen(painter->pen());
            pen.setColor(QColor("red"));
            pen.setWidth(1);
            painter->setPen(pen);

            Edge *e = simMap->getGraph()->getEdge(v1,v2);
            if(e!=NULL) {
                Vertex *v1 = simMap->getGraph()->getVertex(e->v1);
                Vertex *v2 = simMap->getGraph()->getVertex(e->v2);

                if(v1->mapId!=v2->mapId) {
                    cout<<"edge between: "<<v1<<" "<<v2<<endl;

                    drawEdge(e,dispMode,true);

                    mapLabel->setPixmap(*pixmap);
                    mapLabel->adjustSize();
                    scrollArea->resize(mapLabel->size());
                    adjustSize();

                    //showEdge(e);
                }
            }
        }
    }
    else {
        loadMapImage(dispMode);
    }
}


// draw vertices based on display mode using their preloaded poses
void MapWidget::drawVertex(Vertex *v, SimGraph::DispMode d)
{
    if(v!=NULL && painter!=NULL) {
        QPen pen(painter->pen());

        double vertexRadius = getVertexRadius(d);

        if(d==SimGraph::CIRCMODE) {

            v->circPose.boxHeight = vertexRadius;
            v->circPose.boxWidth = vertexRadius;

            if(coloring=="mapId")
                pen.setColor(QColor(colors[v->mapId].c_str())); //mapId is used for merged maps
            else if(coloring=="clusterId")
                pen.setColor(QColor(colors[v->clusterId].c_str())); //clusterId is used for single maps
            else if(coloring=="customVar")
                pen.setColor(QColor(colors[v->customVar].c_str())); //customVar..

            pen.setWidth(2);
            painter->setPen(pen);
            painter->setBrush(pen.color());

            cv::Point2d pt = v->circPose.center;
            QRect rect(pt.x-vertexRadius/2, pt.y-vertexRadius/2, vertexRadius, vertexRadius);
            painter->drawEllipse(rect);

            /*
            // Print vertex label
            pen.setColor(Qt::black);
            pen.setWidth(2);
            p.setPen(pen);
            p.drawText(rect,Qt::AlignCenter, QString(v->label.c_str()));
            */
        }
        else if(d==SimGraph::GRIDMAPMODE)
        {
            if(coloring=="mapId")
                pen.setColor(QColor(colors[v->mapId].c_str())); //mapId is used for merged maps
            else if(coloring=="clusterId")
                pen.setColor(QColor(colors[v->clusterId].c_str())); //clusterId is used for single maps
            else if(coloring=="customVar")
                pen.setColor(QColor(colors[v->customVar].c_str())); //customVar..

            pen.setWidth(2);
            painter->setPen(pen);
            painter->setBrush(Qt::transparent);

            cv::Point2d pt = v->gridmapPose.center;
            painter->drawEllipse(pt.x-vertexRadius/2, pt.y-vertexRadius/2, vertexRadius, vertexRadius);

            v->gridmapPose.boxHeight = vertexRadius;
            v->gridmapPose.boxWidth = vertexRadius;
        }
    }
}


void MapWidget::drawEdge(Edge *e, SimGraph::DispMode d, bool markCenter)
{
    if(e!=NULL && painter!=NULL) {
        QPen pen(painter->pen());

        double edgeLabelRadius = getEdgeLabelRadius(d);

        Vertex* v1 = simMap->getGraph()->getVertex(e->v1);
        Vertex* v2 = simMap->getGraph()->getVertex(e->v2);

        if(d==SimGraph::CIRCMODE) {
            cv::Point2d startPt = v1->circPose.center;
            cv::Point2d endPt = v2->circPose.center;

            e->circPose.boxHeight = edgeLabelRadius;
            e->circPose.boxWidth = edgeLabelRadius;

            if(coloring=="mapId") {
                if(v1->mapId == v2->mapId)
                    pen.setColor(QColor("azure3"));
                else
                    pen.setColor(QColor("blue"));
            }
            else if(coloring=="clusterId") {
                pen.setColor(QColor("azure3"));
            }
            else if(coloring=="customVar") {
                if(e->customVar < 0)
                    pen.setColor(QColor("black"));
                else
                    pen.setColor(QColor("azure3"));
            }
            pen.setWidth(0);
            painter->setPen(pen);

            painter->drawLine(QPointF(startPt.x,startPt.y),QPointF(endPt.x,endPt.y));

            if(markCenter) {
                pen.setColor(Qt::green);
                painter->setPen(pen);
                cv::Point2d center = e->circPose.center;
                painter->drawEllipse(center.x-1, center.y-1, 2, 2);
            }
        }
        else if(d==SimGraph::GRIDMAPMODE) {
            cv::Point2d startPt = v1->gridmapPose.center;
            cv::Point2d endPt = v2->gridmapPose.center;

            e->gridmapPose.boxHeight = edgeLabelRadius;
            e->gridmapPose.boxWidth = edgeLabelRadius;

            if(coloring=="mapId") {
                if(v1->mapId == v2->mapId)
                    pen.setColor(QColor("azure3"));
                else
                    pen.setColor(QColor("blue"));
            }
            else if(coloring=="clusterId") {
                pen.setColor(QColor("azure3"));
            }
            else if(coloring=="customVar") {
                if(e->customVar < 0)
                    pen.setColor(QColor("black"));
                else
                    pen.setColor(QColor("azure3"));
            }
            pen.setWidth(1);
            painter->setPen(pen);

            painter->drawLine(QPointF(startPt.x,startPt.y),QPointF(endPt.x,endPt.y));
            painter->drawEllipse(startPt.x-1, startPt.y-1, 2, 2);
            painter->drawEllipse(endPt.x-1, endPt.y-1, 2, 2);

            if(markCenter) {
                pen.setColor(Qt::green);
                painter->setPen(pen);
                cv::Point2d center = e->gridmapPose.center;
                painter->drawEllipse(center.x-1, center.y-1, 2, 2);
            }
        }
    }
}

// @TODO: vertex radius represents its radius in number of pixels depending on the dispmode
// Need to add zooming capability...
double MapWidget::getVertexRadius(SimGraph::DispMode d) {
    double vertexRadius = 20;
    if(d==SimGraph::CIRCMODE) {
        int N = simMap->numVertices();
        if(N > 1000)
            vertexRadius = 1;
        else if(N > 700)
            vertexRadius = 2;
        else if(N > 400)
            vertexRadius = 3;
        else if(N > 200)
            vertexRadius = 6;
        else if(N > 100)
            vertexRadius = 10;
        else
            vertexRadius = 20;
    }
    else if(d==SimGraph::GRAPHMODE) {
        vertexRadius = 20;
    }

    return vertexRadius;
}


double MapWidget::getEdgeLabelRadius(SimGraph::DispMode d) {
    double edgeLabelRadius = 20;
    if(d==SimGraph::CIRCMODE) {
        edgeLabelRadius = 20;
    }
    else if(d==SimGraph::GRAPHMODE) {
        edgeLabelRadius = 20;
    }
    return edgeLabelRadius;
}




void MapWidget::loadMapImage(SimGraph::DispMode d) {
    if(simMap!=NULL) {

        dispMode = d;

        double vertexRadius = getVertexRadius(dispMode);

        simMap->getGraph()->computeConnectedComponents();

        if(dispMode==SimGraph::GRAPHMODE) {
            // using dot
            QString imagefile;
            simMap->printGraph(imagefile,QString("png"),QString("dot"));
            simMap->getGraph()->updateDisplay(dispMode);
            if(!imagefile.isEmpty()) {
                cout<<"Creating pix"<<endl;
                if(pixmap!=NULL) {
                    painter->end();
                    delete pixmap;
                }
                pixmap = new QPixmap(imagefile);
                painter->begin(pixmap);
                painter->setRenderHint(QPainter::Antialiasing, true);
            }
        }
        else if(dispMode==SimGraph::CIRCMODE) {
            int w,h;
            simMap->getGraph()->computeGraphLayoutCirc(w,h,vertexRadius,vertexRadius*3);
            simMap->getGraph()->updateDisplay(dispMode);

            if(pixmap!=NULL) {
                painter->end();
                delete pixmap;
            }
            pixmap = new QPixmap(w,h);
            pixmap->fill(Qt::white);
            painter->begin(pixmap);
            painter->setRenderHint(QPainter::Antialiasing, true);

            // VERTICES
            vector<Vertex*> *vTable = simMap->getGraph()->getVertexPositions();
            cout<<"Drawing "<<vTable->size()<<" vertices"<<endl;
            for(int i=0; i<vTable->size(); i++) {
                Vertex* v = (*vTable)[i];
                drawVertex(v,SimGraph::CIRCMODE);
            }

            // EDGES
            vector<Edge*> *eTable = simMap->getGraph()->getEdgePositions();
            cout<<"Drawing "<<eTable->size()<<" edges"<<endl;
            int invalids = 0;
            for(int i=0; i<eTable->size(); i++) {
                Edge* e = (*eTable)[i];
                drawEdge(e,SimGraph::CIRCMODE);

                if(e->customVar < 0)
                    invalids++;
            }
            cout<<"Number of invalid edges: "<<invalids<<endl;
        }
        else if(dispMode==SimGraph::GRIDMAPMODE) {
            if(pixmap!=NULL) {
                painter->end();
                delete pixmap;
            }
            pixmap = new QPixmap(QString(appConf.GRIDMAP_FILE.c_str()));
            painter->begin(pixmap);
            painter->setRenderHint(QPainter::Antialiasing, true);
            simMap->getGraph()->updateDisplay(dispMode);

            // EDGES
            vector<Edge*> *eTable = simMap->getGraph()->getEdgePositions();
            cout<<"Drawing "<<eTable->size()<<" edges"<<endl;
            for(int i=0; i<eTable->size(); i++) {
                Edge *e = (*eTable)[i];
                drawEdge(e,SimGraph::GRIDMAPMODE);
            }

            // VERTICES
            vector<Vertex*> *vTable = simMap->getGraph()->getVertexPositions();
            cout<<"Drawing "<<vTable->size()<<" vertices"<<endl;
            for(int i=0; i<vTable->size(); i++) {
                Vertex* v = (*vTable)[i];
                drawVertex(v,SimGraph::GRIDMAPMODE);
            }
        }

        // @TODO: this mode is not finalized yet!
        else if(dispMode==SimGraph::MATRIXMODE) {
            int vertexWidth = 5;
            int borderWidth = 10;
            int margin = 5;
            int lineWidth = 2;

            simMap->getGraph()->updateDisplay(dispMode);
            vector<Vertex*> *vTable = simMap->getGraph()->getVertexPositions();
            vector<Edge*> *eTable = simMap->getGraph()->getEdgePositions();

            int numVertices = vTable->size();

            int w = numVertices*vertexWidth + 2*margin + borderWidth + lineWidth;
            if(pixmap!=NULL) {
                painter->end();
                delete pixmap;
            }
            pixmap = new QPixmap(w,w);
            pixmap->fill(Qt::white);
            painter->begin(pixmap);
            painter->setRenderHint(QPainter::Antialiasing, true);

            QColor c(QColor("gray1"));
            QPen pen(painter->pen());
            pen.setColor(c);
            pen.setWidth(lineWidth);
            painter->setPen(pen);
            painter->drawLine(QPoint(margin+borderWidth,margin),QPoint(margin+borderWidth,w-margin));
            painter->drawLine(QPoint(margin,margin+borderWidth),QPoint(w-margin,margin+borderWidth));

            // VERTICES
            for(int i=0; i<vTable->size(); i++) {
                Vertex *v = (*vTable)[i];
                QColor c(colors[v->mapId].c_str());
                pen.setColor(c);
                pen.setWidth(vertexWidth);
                painter->setPen(pen);
                painter->setBrush(c);

                int x,y;
                x = margin+borderWidth+lineWidth+i*vertexWidth;
                y = margin;
                //p.drawLine(QPoint(x,y),QPoint(x,y+borderWidth));

                x = margin;
                y = margin+borderWidth+lineWidth+i*vertexWidth;
                //p.drawLine(QPoint(x,y),QPoint(x+borderWidth,y));
                //p.drawRect(QRect(QPoint(margin+borderWidth+lineWidth+1+i*vertexWidth,margin),QSize(vertexWidth,borderWidth)));
                //p.drawRect(QRect(QPoint(margin,margin+borderWidth+lineWidth+1+i*vertexWidth),QSize(borderWidth,vertexWidth)));
            }
        }

        mapLabel->setPixmap(*pixmap);
        mapLabel->adjustSize();
        scrollArea->resize(mapLabel->size());
        adjustSize();
    }
}

void MapWidget::switchToCircMode() {
    loadMapImage(SimGraph::CIRCMODE);
}
void MapWidget::switchToGraphMode() {
    loadMapImage(SimGraph::GRAPHMODE);
}
void MapWidget::switchToGridmapMode() {
    loadMapImage(SimGraph::GRIDMAPMODE);
}
void MapWidget::switchToMatrixMode() {
    loadMapImage(SimGraph::MATRIXMODE);
}

void MapWidget::resetVertexLabels() {
    if(simMap!=NULL) {
        simMap->getGraph()->resetVertexLabels();
        loadMapImage();
    }
}


void MapWidget::attachWidget() {
    QMdiSubWindow *s = mdiArea->addSubWindow(this);
    s->setWindowIcon(QIcon(ICON_MAP));
    s->setWindowTitle(getTitle());
    s->resize(size()+QSize(10,30));
    s->show();
    attachWidgetAct->setEnabled(false);
    detachWidgetAct->setEnabled(true);
}

void MapWidget::detachWidget() {
    QMdiSubWindow *s = mdiArea->currentSubWindow();
    mdiArea->removeSubWindow(this);
    s->close();
    this->show();
    attachWidgetAct->setEnabled(true);
    detachWidgetAct->setEnabled(false);
}







bool MapWidget::openMap(QString fname) {
    filename = fname;
    QFileInfo fi(filename);
    setTitle(fi.fileName());
    simMap = new SimMap(fname.toStdString());
    return true;
}


bool MapWidget::saveMap(QString fname) {
    if(simMap!=NULL) {
        if(!fname.isEmpty()) {
            setFilename(fname);
            QFileInfo fi(fname);
            setTitle(fi.fileName());
        }
        return simMap->writeMap(filename.toStdString());
    }
}


// save the map image to a new file
void MapWidget::exportMap(QString fullpath) {
    mapLabel->pixmap()->save(fullpath);
}


// build map from scratch using the selected images and options
void MapWidget::buildMap(QStringList imagefiles, bool isForced, bool addEdges)
{
    if(simMap!=NULL) {
        delete simMap;
        simMap = NULL;
    }
    simMap = new SimMap;
    filename = "";
    title = "";

    emit Busy();
    ProgressBar progressBar;
    progressBar.setWindowTitle("Map building progress");
    progressBar.move(QApplication::desktop()->availableGeometry(this).center() - progressBar.rect().center());
    progressBar.show();
    connect(&progressBar,SIGNAL(dialogClosed()),this,SLOT(stop()));

    int count = 0;
    double progress = 0.0;
    simMap->clearTrackingList();
    struct timeval time_start,time_end;
    gettimeofday(&time_start,NULL);
    foreach(QString s, imagefiles) {
        simMap->addImage(s,-1,isForced,appConf.TRACKINGMODE,addEdges);
        progress = 100.0*(++count)/(double)imagefiles.size();
        progressBar.setValue((int)progress);

        qApp->processEvents();
        if(processStop)
            break;
    }
    gettimeofday(&time_end,NULL);
    double elapsed = (time_end.tv_sec -time_start.tv_sec) + (time_end.tv_usec - time_start.tv_usec)/1000000.0;
    Debug(1)<<"MAP BUILD TIME: "<<elapsed<<endl;

    simMap->clearTrackingList();
    processStop = false;
    setFilename("");
    setTitle("New map");

    simMap->getGraph()->computeConnectedComponents();
    simMap->updateStats();

    emit notBusy();

#if _TIMING==1
    //cout<<"TIMING"<<endl;
    for(int i=0; i<simMap->timing.size(); i++) {
        cout<<simMap->timing[i]<<endl;
    }
#endif
}


// Add images to the map
void MapWidget::addImages(QStringList imagefiles, bool isForced, bool addEdges)
{
    if(simMap==NULL) {
        simMap = new SimMap;
    }
    int count = 0;
    double progress = 0.0;
    foreach(QString s, imagefiles) {
        simMap->addImage(s,-1,isForced,appConf.TRACKINGMODE, addEdges);
        progress = (++count)/(double)imagefiles.size();
    }
}

bool MapWidget::addEdge(int v1, int v2)
{
    if(simMap==NULL)
        return false;

    return simMap->addEdge(v1,v2);
}


bool MapWidget::removeVertex(int v)
{
    if(simMap==NULL)
        return false;

    return simMap->removeVertex(v);
}

bool MapWidget::removeConnectedComp(int c)
{
    if(simMap==NULL)
        return false;

    return simMap->removeConnectedComp(c);
}


bool MapWidget::removeEdge(int v1, int v2)
{
    if(simMap==NULL)
        return false;

    return simMap->removeEdge(v1,v2);
}

bool MapWidget::removeEdges(vector<pair<int,int> > lim1, vector<pair<int,int> > lim2)
{
    if(simMap==NULL)
        return false;

    return simMap->removeEdges(lim1,lim2);
}


// reserved for testing some new functions and debugging...
void MapWidget::test() {

    if(simMap!=NULL)
        simMap->getGraph()->test();

    /*
    SimMap* m1 = new SimMap("/home/gorkem/code/VisApp-build/maps/Run19_all_mapimprove.map");
    SimMap* m2 = new SimMap("/home/gorkem/code/VisApp-build/maps/Run23_all.map");
    SimMap::test2(m1,m2);
    QStringList filenames = QFileDialog::getOpenFileNames(this,
                                                          tr("Open image"),".",
                                                          tr("All image files (*.ppm *.pgm)\n"
                                                             "PNG images (*.png)\n"
                                                             "JPG images (*.jpg)\n"
                                                             "PPM images (*.ppm)\n"
                                                             "PGM images (*.pgm)"));
    */
}


// for testing purposes
void MapWidget::writeSimMatrixToFile()
{
    if(simMap!=NULL) {
        simMap->getGraph()->writeSimMatrix("simMatrix.txt");
    }
}

