#ifndef MAPWIDGET_H
#define MAPWIDGET_H

#include "simmap.h"
#include <QWidget>
#include <QLabel>
#include "imagewidget.h"
#include <QEvent>
#include <QMdiArea>
#include <QScrollArea>

//! widget to load a map providing several options accessable through right click menu
class MapWidget : public QWidget
{
    Q_OBJECT
public:
    MapWidget(QWidget *parent=0);
    MapWidget(QString filename, QWidget *parent=0);
    bool openMap(QString filename);
    bool saveMap(QString filename="");
    void exportMap(QString filename);
    void buildMap(QStringList imagefiles, bool isForced=false, bool addEdges=true);
    void addImages(QStringList imagefiles, bool isForced=false, bool addEdges=true);
    bool addEdge(int v1, int v2);
    bool removeVertex(int v);
    bool removeEdge(int v1, int v2);

    // advanced method to remove edges using min-max id limits
    bool removeEdges(vector<pair<int,int> > lim1, vector<pair<int,int> > lim2);

    // v is the size of the connected components that will be removed
    bool removeConnectedComp(int v);

    // load map image using the display mode (circular, graph, or gridmap)
    void loadMapImage(SimGraph::DispMode d=SimGraph::CIRCMODE);

    SimMap* getMap() { return simMap; }
    void setMdiPtr(QMdiArea* m);

    QString getFilename();
    void setFilename(QString fname);
    QString getTitle();
    void setTitle(QString t);

    //double getProgress() { return progress; }

public slots:
    void test();
signals:
    void closing();
    void Busy();
    void notBusy();
protected:
    void closeEvent(QCloseEvent *);
    void mousePressEvent(QMouseEvent *);
    void resizeEvent(QResizeEvent *);
private slots:
    void mapInfo();
    void writeStatsToFile(); // writes map stats to a file
    void writeSimMatrixToFile(); // writes similarity matrix to a file
    void showTiming(); // shows timing of the the mapping process
    void computeScore();

    // switch visualization mode
    void switchToCircMode();
    void switchToGraphMode();
    void switchToGridmapMode();
    void switchToMatrixMode();

    void stop();
    void resetVertexLabels();
    void attachWidget();
    void detachWidget();
    void updateActions();
    void changeColoring(QString s);
private:
    SimMap *simMap;
    QString filename;
    QString title;
    QLabel *mapLabel;
    QPixmap *pixmap;
    //ImageWidget* mapImage;
    QMdiArea *mdiArea;
    QAction *mapInfoAct;
    QAction *statsToFileAct;
    QAction *simMatrixToFileAct;
    QAction *showTimingAct;
    QAction *computeScoreAct;
    QAction *graphModeAct;
    QAction *circModeAct;
    QAction *matrixModeAct;
    QAction *gridmapModeAct;
    QAction *resetVertexLabelsAct;
    QAction *detachWidgetAct;
    QAction *attachWidgetAct;
    QAction *colorByMapId;
    QAction *colorByClusterId;
    QAction *colorByCustomVar;
    QAction *colorVertex;
    QAction *colorEdge;

    QString coloring;

    QScrollArea *scrollArea;

    void createActions();

    void showVertex(Vertex*);
    void showEdge(Edge*);
    void drawVertex(Vertex *v, SimGraph::DispMode);
    void drawEdge(Edge*, SimGraph::DispMode, bool markCenter=false);
    QPainter *painter;

    double getVertexRadius(SimGraph::DispMode);
    double getEdgeLabelRadius(SimGraph::DispMode);

    //double progress;
    bool processStop;

    //int dispMode; // graph style or gridmap style
    SimGraph::DispMode dispMode;
};




#endif // MAPWIDGET_H
